@@ -35,14 +35,15 @@ class TreeView(gtk.TreeView):
             field = record[column.name]
             field.state_set(record, states=('readonly', 'invisible'))
             invisible = field.get_state_attrs(record).get('invisible', False)
+            if not column.get_visible():
+                invisible = True
             if editable:
                 readonly = widget.attrs.get('readonly',
                     field.get_state_attrs(record).get('readonly', False))
             else:
                 readonly = False
             if not (invisible or readonly):
-                break
-        return column
+                return column
 
     def prev_column(self, path, column=None, editable=True):
         return self.next_column(path, column=column, editable=editable,
